@@ -25,6 +25,7 @@
 
 #include <stdint.h>
 #include <errno.h>
+#include <stdbool.h>
 #include <string.h>
 #include <sys/socket.h>
 
@@ -63,7 +64,7 @@ extern int mifare_write(uint32_t adapter_idx, uint32_t target_idx,
 
 #define TAG_DATA_CC(data) ((data) + 12)
 #define TAG_DATA_LENGTH(cc) ((cc)[2] * 8)
-#define TAG_DATA_NFC(cc) ((cc)[0] & TYPE2_MAGIC)
+#define TAG_DATA_NFC(cc) ((cc)[0])
 
 #define TYPE2_NOWRITE_ACCESS	0x0F
 #define TYPE2_READWRITE_ACCESS	0x00
@@ -110,10 +111,10 @@ static int t2_cookie_release(int err, void *data)
 
 	DBG("%p", cookie);
 
-	if (cookie == NULL)
+	if (!cookie)
 		return err;
 
-	if (cookie->cb != NULL)
+	if (cookie->cb)
 		cookie->cb(cookie->adapter_idx, cookie->target_idx, err);
 
 	if (cookie->ndef)
@@ -121,7 +122,6 @@ static int t2_cookie_release(int err, void *data)
 
 	g_free(cookie->ndef);
 	g_free(cookie);
-	cookie = NULL;
 
 	return err;
 }
@@ -232,13 +232,13 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 		goto out_err;
 
 	tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
-	if (tag == NULL) {
+	if (!tag) {
 		err = -ENOMEM;
 		goto out_err;
 	}
 
 	t2_tag = g_try_malloc0(sizeof(struct type2_tag));
-	if (t2_tag == NULL) {
+	if (!t2_tag) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -255,7 +255,7 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 
 	near_tag_set_memory_layout(tag, NEAR_TAG_MEMORY_STATIC);
 
-	if (TAG_DATA_NFC(cc) == 0) {
+	if (TAG_DATA_NFC(cc) != TYPE2_MAGIC) {
 		DBG("Mark as blank tag");
 		near_tag_set_blank(tag, TRUE);
 	} else {
@@ -292,7 +292,7 @@ static int nfctype2_read_meta(uint32_t adapter_idx, uint32_t target_idx,
 	cmd.block = META_BLOCK_START;
 
 	cookie = g_try_malloc0(sizeof(struct t2_cookie));
-	if (cookie == NULL)
+	if (!cookie)
 		return -ENOMEM;
 
 	cookie->adapter_idx = adapter_idx;
@@ -381,9 +381,9 @@ static int data_write(uint32_t adapter_idx, uint32_t target_idx,
 	DBG("");
 
 	cookie = g_try_malloc0(sizeof(struct t2_cookie));
-	if (cookie == NULL) {
+	if (!cookie) {
 		err = -ENOMEM;
-		if (cb != NULL)
+		if (cb)
 			cb(adapter_idx, target_idx, err);
 		return err;
 	}
@@ -415,13 +415,13 @@ static int nfctype2_write(uint32_t adapter_idx, uint32_t target_idx,
 
 	DBG("");
 
-	if (ndef == NULL || cb == NULL) {
+	if (!ndef || !cb) {
 		err = -EINVAL;
 		goto out_err;
 	}
 
 	tag = near_tag_get_tag(adapter_idx, target_idx);
-	if (tag == NULL) {
+	if (!tag) {
 		err = -EINVAL;
 		goto out_err;
 	}
@@ -461,7 +461,7 @@ static int nfctype2_write(uint32_t adapter_idx, uint32_t target_idx,
 	return 0;
 
 out_err:
-	if (cb != NULL)
+	if (cb)
 		cb(adapter_idx, target_idx, err);
 
 	return err;
@@ -497,7 +497,7 @@ static int nfctype2_check_presence(uint32_t adapter_idx, uint32_t target_idx,
 		cmd.block = META_BLOCK_START;
 
 		cookie = g_try_malloc0(sizeof(struct t2_cookie));
-		if (cookie == NULL)
+		if (!cookie)
 			return -ENOMEM;
 
 		cookie->adapter_idx = adapter_idx;
@@ -535,7 +535,7 @@ static int format_resp(uint8_t *resp, int length, void *data)
 	}
 
 	tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
-	if (tag == NULL) {
+	if (!tag) {
 		err = -EINVAL;
 		goto out_err;
 	}
@@ -561,7 +561,7 @@ static int nfctype2_format(uint32_t adapter_idx, uint32_t target_idx,
 	DBG("");
 
 	tag = near_tag_get_tag(adapter_idx, target_idx);
-	if (tag == NULL)
+	if (!tag)
 		return -EINVAL;
 
 
@@ -576,7 +576,7 @@ static int nfctype2_format(uint32_t adapter_idx, uint32_t target_idx,
 	cc_ndef = g_try_malloc0(sizeof(struct near_ndef_message));
 	cookie = g_try_malloc0(sizeof(struct t2_cookie));
 
-	if (t2_cc == NULL || cc_ndef == NULL || cookie == NULL) {
+	if (!t2_cc || !cc_ndef || !cookie) {
 		err = -ENOMEM;
 		goto out_err;
 	}
