@@ -20,6 +20,7 @@
  */
 
 #include <stdint.h>
+#include <stdbool.h>
 #include <sys/socket.h>
 
 #include <linux/socket.h>
@@ -44,6 +45,7 @@ void __near_dbus_cleanup(void);
 
 DBusMessage *__near_error_failed(DBusMessage *msg, int errnum);
 DBusMessage *__near_error_invalid_arguments(DBusMessage *msg);
+DBusMessage *__near_error_out_of_memory(DBusMessage *msg);
 DBusMessage *__near_error_permission_denied(DBusMessage *msg);
 DBusMessage *__near_error_passphrase_required(DBusMessage *msg);
 DBusMessage *__near_error_not_registered(DBusMessage *msg);
@@ -63,9 +65,10 @@ DBusMessage *__near_error_operation_aborted(DBusMessage *msg);
 DBusMessage *__near_error_operation_timeout(DBusMessage *msg);
 DBusMessage *__near_error_invalid_service(DBusMessage *msg);
 DBusMessage *__near_error_invalid_property(DBusMessage *msg);
+DBusMessage *__near_error_io_error(DBusMessage *msg);
 
 int __near_manager_adapter_add(uint32_t idx, const char *name,
-			uint32_t protocols, near_bool_t powered);
+			uint32_t protocols, bool powered);
 void __near_manager_adapter_remove(uint32_t idx);
 int __near_manager_init(DBusConnection *conn);
 void __near_manager_cleanup(void);
@@ -73,7 +76,7 @@ void __near_manager_cleanup(void);
 #include <near/adapter.h>
 
 struct near_adapter *__near_adapter_create(uint32_t idx,
-		const char *name, uint32_t protocols, near_bool_t powered);
+		const char *name, uint32_t protocols, bool powered);
 void __near_adapter_destroy(struct near_adapter *adapter);
 const char *__near_adapter_get_path(struct near_adapter *adapter);
 struct near_adapter *__near_adapter_get(uint32_t idx);
@@ -81,16 +84,16 @@ int __near_adapter_add(struct near_adapter *adapter);
 void __near_adapter_remove(struct near_adapter *adapter);
 int __near_adapter_add_target(uint32_t idx, uint32_t target_idx,
 			uint32_t protocols, uint16_t sens_res, uint8_t sel_res,
-			uint8_t *nfcid, uint8_t nfcid_len);
+			uint8_t *nfcid, uint8_t nfcid_len,
+			uint8_t iso15693_dsfid,
+			uint8_t iso15693_uid_len, uint8_t *iso15693_uid);
 int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx);
+int __near_adapter_get_targets_done(uint32_t idx);
 int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len);
 int __near_adapter_remove_device(uint32_t idx);
-int __near_adapter_set_dep_state(uint32_t idx, near_bool_t dep);
-near_bool_t __near_adapter_get_dep_state(uint32_t idx);
-void __near_adapter_tags_changed(uint32_t adapter_idx);
-void __near_adapter_devices_changed(uint32_t adapter_idx);
+int __near_adapter_set_dep_state(uint32_t idx, bool dep);
+bool __near_adapter_get_dep_state(uint32_t idx);
 void __near_adapter_listen(struct near_device_driver *driver);
-void __near_adapter_list(DBusMessageIter *iter, void *user_data);
 void __near_adapter_start_check_presence(uint32_t adapter_idx, uint32_t target_idx);
 void __near_adapter_stop_check_presence(uint32_t adapter_idx, uint32_t target_idx);
 int __near_adapter_init(void);
@@ -107,6 +110,7 @@ void __near_ndef_record_free(struct near_ndef_record *record);
 char *__near_ndef_record_get_path(struct near_ndef_record *record);
 char *__near_ndef_record_get_type(struct near_ndef_record *record);
 uint8_t *__near_ndef_record_get_data(struct near_ndef_record *record, size_t *len);
+uint8_t *__near_ndef_record_get_payload(struct near_ndef_record *record, size_t *len);
 void __near_ndef_append_records(DBusMessageIter *iter, GList *record);
 const char *__near_ndef_get_uri_prefix(uint8_t id);
 struct near_ndef_message *__ndef_build_from_message(DBusMessage *msg);
@@ -120,10 +124,13 @@ void __near_snep_core_cleanup(void);
 
 int __near_tag_init(void);
 void __near_tag_cleanup(void);
-struct near_tag *__near_tag_add(uint32_t idx, uint32_t target_idx,
+struct near_tag *__near_tag_add(uint32_t adapter_idx, uint32_t target_idx,
 				uint32_t protocols,
 				uint16_t sens_res, uint8_t sel_res,
-				uint8_t *nfcid, uint8_t nfcid_len);
+				uint8_t *nfcid, uint8_t nfcid_len,
+				uint8_t iso15693_dsfid,
+				uint8_t iso15693_uid_len,
+				uint8_t *iso15693_uid);
 void __near_tag_remove(struct near_tag *tag);
 const char *__near_tag_get_path(struct near_tag *tag);
 uint32_t __near_tag_get_type(struct near_tag *tag);
@@ -143,6 +150,7 @@ uint32_t __neard_device_get_idx(struct near_device *device);
 struct near_device *__near_device_add(uint32_t idx, uint32_t target_idx,
 					uint8_t *nfcid, uint8_t nfcid_len);
 void __near_device_remove(struct near_device *device);
+bool __near_device_register_interface(struct near_device *device);
 int __near_device_listen(struct near_device *device, near_device_io_cb cb);
 int __near_device_push(struct near_device *device,
 			struct near_ndef_message *ndef, char *service_name,
@@ -157,7 +165,7 @@ int __near_netlink_stop_poll(int idx);
 int __near_netlink_dep_link_up(uint32_t idx, uint32_t target_idx,
 				uint8_t comm_mode, uint8_t rf_mode);
 int __near_netlink_dep_link_down(uint32_t idx);
-int __near_netlink_adapter_enable(int idx, near_bool_t enable);
+int __near_netlink_adapter_enable(int idx, bool enable);
 int __near_netlink_init(void);
 void __near_netlink_cleanup(void);
 
@@ -231,20 +239,12 @@ void __near_bluetooth_cleanup(void);
 void __near_bluetooth_legacy_start(void);
 void __near_bluetooth_legacy_stop(void);
 int __near_bluetooth_parse_oob_record(struct carrier_data *data,
-					uint16_t *properties, near_bool_t pair);
+					uint16_t *properties, bool pair);
 int __near_bluetooth_pair(void *data);
 struct carrier_data *__near_bluetooth_local_get_properties(uint16_t mime_props);
 
 void __near_agent_ndef_parse_records(GList *records);
-int __near_agent_ndef_register(const char *sender, const char *path,
-						const char *record_type);
-int __near_agent_ndef_unregister(const char *sender, const char *path,
-						const char *record_type);
-int __near_agent_handover_register(const char *sender, const char *path,
-					const char *carrier);
-int __near_agent_handover_unregister(const char *sender, const char *path,
-					const char *carrier);
-near_bool_t __near_agent_handover_registered(enum ho_agent_carrier carrier);
+bool __near_agent_handover_registered(enum ho_agent_carrier carrier);
 
 struct carrier_data *__near_agent_handover_request_data(
 					enum ho_agent_carrier carrier,
