@@ -52,6 +52,9 @@ struct near_tag {
 	uint8_t nfcid[NFC_MAX_NFCID1_LEN];
 	uint8_t nfcid_len;
 
+	uint8_t iso15693_dsfid;
+	uint8_t iso15693_uid[NFC_MAX_ISO15693_UID_LEN];
+
 	size_t data_length;
 	uint8_t *data;
 
@@ -69,8 +72,15 @@ struct near_tag {
 	struct {
 		uint16_t max_ndef_size;
 		uint16_t c_apdu_max_size;
+		uint16_t r_apdu_max_size;
+		uint16_t file_id;
 	} t4;
 
+	struct {
+		uint8_t blk_size;
+		uint8_t num_blks;
+	} t5;
+
 	DBusMessage *write_msg; /* Pending write message */
 	struct near_ndef_message *write_ndef;
 };
@@ -138,7 +148,15 @@ static const char *type_string(struct near_tag *tag)
 		break;
 
 	case NFC_PROTO_ISO14443:
-		type = "Type 4";
+		type = "Type 4A";
+		break;
+
+	case NFC_PROTO_ISO14443_B:
+		type = "Type 4B";
+		break;
+
+	case NFC_PROTO_ISO15693:
+		type = "Type 5";
 		break;
 
 	default:
@@ -170,9 +188,14 @@ static const char *protocol_string(struct near_tag *tag)
 		break;
 
 	case NFC_PROTO_ISO14443_MASK:
+	case NFC_PROTO_ISO14443_B_MASK:
 		protocol = "ISO-DEP";
 		break;
 
+	case NFC_PROTO_ISO15693_MASK:
+		protocol = "ISO-15693";
+		break;
+
 	default:
 		near_error("Unknown tag protocol 0x%x", tag->protocol);
 		protocol = NULL;
@@ -364,6 +387,7 @@ static DBusMessage *write_ndef(DBusConnection *conn,
 	switch (tag->type) {
 	case NFC_PROTO_JEWEL:
 	case NFC_PROTO_MIFARE:
+	case NFC_PROTO_ISO15693:
 		if (ndef->length < 0xff)
 			tlv_len_size = 3;
 		else
@@ -416,6 +440,7 @@ static DBusMessage *write_ndef(DBusConnection *conn,
 		break;
 
 	case NFC_PROTO_ISO14443:
+	case NFC_PROTO_ISO14443_B:
 		ndef_with_header = g_try_malloc0(sizeof(
 					struct near_ndef_message));
 		if (!ndef_with_header)
@@ -582,6 +607,10 @@ static void set_tag_type(struct near_tag *tag,
 		tag->type = NFC_PROTO_ISO14443_B;
 		break;
 
+	case NFC_PROTO_ISO15693_MASK:
+		tag->type = NFC_PROTO_ISO15693;
+		break;
+
 	default:
 		tag->type = NFC_PROTO_MAX;
 		break;
@@ -594,7 +623,9 @@ static int tag_initialize(struct near_tag *tag,
 			uint32_t adapter_idx, uint32_t target_idx,
 			uint32_t protocols,
 			uint16_t sens_res, uint8_t sel_res,
-			uint8_t *nfcid, uint8_t nfcid_len)
+			uint8_t *nfcid, uint8_t nfcid_len,
+			uint8_t iso15693_dsfid,
+			uint8_t iso15693_uid_len, uint8_t *iso15693_uid)
 {
 	DBG("");
 
@@ -608,9 +639,12 @@ static int tag_initialize(struct near_tag *tag,
 	tag->next_record = 0;
 	tag->readonly = false;
 
-	if (nfcid_len <= NFC_MAX_NFCID1_LEN) {
+	if (nfcid_len && nfcid_len <= NFC_MAX_NFCID1_LEN) {
 		tag->nfcid_len = nfcid_len;
 		memcpy(tag->nfcid, nfcid, nfcid_len);
+	} else if (iso15693_uid_len) {
+		tag->iso15693_dsfid = iso15693_dsfid;
+		memcpy(tag->iso15693_uid, iso15693_uid, iso15693_uid_len);
 	}
 
 	set_tag_type(tag, sens_res, sel_res);
@@ -621,7 +655,9 @@ static int tag_initialize(struct near_tag *tag,
 struct near_tag *__near_tag_add(uint32_t adapter_idx, uint32_t target_idx,
 				uint32_t protocols,
 				uint16_t sens_res, uint8_t sel_res,
-				uint8_t *nfcid, uint8_t nfcid_len)
+				uint8_t *nfcid, uint8_t nfcid_len,
+				uint8_t iso15693_dsfid,
+				uint8_t iso15693_uid_len, uint8_t *iso15693_uid)
 {
 	struct near_tag *tag;
 	char *path;
@@ -637,7 +673,9 @@ struct near_tag *__near_tag_add(uint32_t adapter_idx, uint32_t target_idx,
 	if (tag_initialize(tag, adapter_idx, target_idx,
 				protocols,
 				sens_res, sel_res,
-				nfcid, nfcid_len) < 0) {
+				nfcid, nfcid_len,
+				iso15693_dsfid,
+				iso15693_uid_len, iso15693_uid) < 0) {
 		g_free(tag);
 		return NULL;
 	}
@@ -739,6 +777,49 @@ int near_tag_set_nfcid(uint32_t adapter_idx, uint32_t target_idx,
 	return 0;
 }
 
+uint8_t *near_tag_get_iso15693_dsfid(uint32_t adapter_idx, uint32_t target_idx)
+{
+	struct near_tag *tag;
+	uint8_t *iso15693_dsfid;
+
+	tag = near_tag_get_tag(adapter_idx, target_idx);
+	if (!tag)
+		goto fail;
+
+	iso15693_dsfid = g_try_malloc0(NFC_MAX_ISO15693_DSFID_LEN);
+	if (!iso15693_dsfid)
+		goto fail;
+
+	memcpy(iso15693_dsfid, &tag->iso15693_dsfid,
+		NFC_MAX_ISO15693_DSFID_LEN);
+
+	return iso15693_dsfid;
+
+fail:
+	return NULL;
+}
+
+uint8_t *near_tag_get_iso15693_uid(uint32_t adapter_idx, uint32_t target_idx)
+{
+	struct near_tag *tag;
+	uint8_t *iso15693_uid;
+
+	tag = near_tag_get_tag(adapter_idx, target_idx);
+	if (!tag)
+		goto fail;
+
+	iso15693_uid = g_try_malloc0(NFC_MAX_ISO15693_UID_LEN);
+	if (!iso15693_uid)
+		goto fail;
+
+	memcpy(iso15693_uid, tag->iso15693_uid, NFC_MAX_ISO15693_UID_LEN);
+
+	return iso15693_uid;
+
+fail:
+	return NULL;
+}
+
 int near_tag_add_data(uint32_t adapter_idx, uint32_t target_idx,
 			uint8_t *data, size_t data_length)
 {
@@ -888,6 +969,38 @@ uint16_t near_tag_get_c_apdu_max_size(struct near_tag *tag)
 	return tag->t4.c_apdu_max_size;
 }
 
+void near_tag_set_r_apdu_max_size(struct near_tag *tag, uint16_t size)
+{
+	if (!tag)
+		return;
+
+	tag->t4.r_apdu_max_size = size;
+}
+
+uint16_t near_tag_get_r_apdu_max_size(struct near_tag *tag)
+{
+	if (!tag)
+		return 0;
+
+	return tag->t4.r_apdu_max_size;
+}
+
+void near_tag_set_file_id(struct near_tag *tag, uint16_t file_id)
+{
+	if (!tag)
+		return;
+
+	tag->t4.file_id = file_id;
+}
+
+uint16_t near_tag_get_file_id(struct near_tag *tag)
+{
+	if (!tag)
+		return 0;
+
+	return tag->t4.file_id;
+}
+
 void near_tag_set_idm(struct near_tag *tag, uint8_t *idm, uint8_t len)
 {
 	if (!tag || len > TYPE3_IDM_LEN)
@@ -940,6 +1053,26 @@ uint8_t near_tag_get_ic_type(struct near_tag *tag)
 	return tag->t3.ic_type;
 }
 
+uint8_t near_tag_get_blk_size(struct near_tag *tag)
+{
+	return tag->t5.blk_size;
+}
+
+void near_tag_set_blk_size(struct near_tag *tag, uint8_t blk_size)
+{
+	tag->t5.blk_size = blk_size;
+}
+
+uint8_t near_tag_get_num_blks(struct near_tag *tag)
+{
+	return tag->t5.num_blks;
+}
+
+void near_tag_set_num_blks(struct near_tag *tag, uint8_t num_blks)
+{
+	tag->t5.num_blks = num_blks;
+}
+
 static gint cmp_prio(gconstpointer a, gconstpointer b)
 {
 	const struct near_tag_driver *driver1 = a;
