@@ -30,6 +30,8 @@
 #include <sys/socket.h>
 #include <glib.h>
 
+#include <netlink/genl/genl.h>
+
 #include <near/nfc_copy.h>
 
 #include "nfctool.h"
@@ -46,7 +48,7 @@ static GMainLoop *main_loop = NULL;
 static int nfctool_poll_cb(guint8 cmd, guint32 idx, gpointer data);
 static int nfctool_snl_cb(guint8 cmd, guint32 idx, gpointer data);
 
-static void nfctool_quit(gboolean force);
+static void nfctool_quit(bool force);
 
 static gchar *nfctool_poll_mode_str(int mode)
 {
@@ -67,7 +69,7 @@ static int nfctool_start_poll(void)
 
 	adapter = adapter_get(opts.adapter_idx);
 
-	if (adapter == NULL) {
+	if (!adapter) {
 		print_error("Invalid adapter index: %d", opts.adapter_idx);
 
 		return -ENODEV;
@@ -135,6 +137,66 @@ static int nfctool_snl_send_request(struct nfc_adapter *adapter)
 	return err;
 }
 
+static int nfctool_set_powered(bool powered)
+{
+	struct nfc_adapter *adapter;
+	int err;
+
+	adapter = adapter_get(opts.adapter_idx);
+	if (!adapter)
+		return -ENODEV;
+
+	err = nl_set_powered(adapter, powered);
+
+	if (err == 0)
+		adapter->powered = powered;
+
+	return err;
+}
+
+static int nfctool_fw_download_cb(guint8 cmd, guint32 adapter_idx,
+				  gpointer data)
+{
+	int err;
+	gchar *fw_filename;
+	struct nlattr **nl_attr = data;
+
+	if (nl_attr[NFC_ATTR_FIRMWARE_DOWNLOAD_STATUS] != NULL)
+		err = nla_get_u32(nl_attr[NFC_ATTR_FIRMWARE_DOWNLOAD_STATUS]);
+	else
+		err = -ENOTSUP;
+
+	if (nl_attr[NFC_ATTR_FIRMWARE_NAME] != NULL)
+		fw_filename = nla_get_string(nl_attr[NFC_ATTR_FIRMWARE_NAME]);
+	else
+		fw_filename = "UNKNOWN";
+
+	printf("Firmware download operation for %s terminated with status %s\n",
+	       fw_filename, err ? strerror(-err) : "OK");
+
+	nfctool_quit(false);
+
+	return 0;
+}
+
+static int nfctool_fw_download(gchar *fw_filename)
+{
+	struct nfc_adapter *adapter;
+	int err;
+
+	adapter = adapter_get(opts.adapter_idx);
+	if (!adapter)
+		return -ENODEV;
+
+	nl_add_event_handler(NFC_CMD_FW_DOWNLOAD, nfctool_fw_download_cb);
+
+	err = nl_fw_download(adapter, fw_filename);
+	if (err)
+		print_error("Firmware download failed: %s", strerror(-err));
+
+	return err;
+}
+
 static int nfctool_dep_link_up_cb(guint8 cmd, guint32 idx, gpointer data)
 {
 	struct nfc_adapter *adapter;
@@ -160,9 +222,9 @@ static int nfctool_dep_link_down_cb(guint8 cmd, guint32 idx, gpointer data)
 
 	printf("Link is DOWN for adapter nfc%d\n\n", idx);
 
-	opts.snl = FALSE;
+	opts.snl = false;
 
-	nfctool_quit(FALSE);
+	nfctool_quit(false);
 
 	return 0;
 }
@@ -189,7 +251,7 @@ static int nfctool_snl(void)
 	if (adapter->rf_mode == NFC_RF_NONE) {
 		print_error("Can't send SNL request: No active link");
 
-		opts.snl = FALSE;
+		opts.snl = false;
 
 		return -ENOLINK;
 	}
@@ -214,7 +276,7 @@ static int nfctool_targets_found(guint32 adapter_idx)
 
 	adapter = adapter_get(adapter_idx);
 
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	err = nl_get_targets(adapter);
@@ -254,7 +316,7 @@ static int nfctool_poll_cb(guint8 cmd, guint32 idx, gpointer data)
 		break;
 	}
 
-	nfctool_quit(FALSE);
+	nfctool_quit(false);
 
 	return err;
 }
@@ -270,7 +332,7 @@ static void nfctool_print_and_remove_snl(struct nfc_snl *sdres,
 		elem = g_slist_find_custom(opts.snl_list, sdres->uri,
 					   (GCompareFunc)g_strcmp0);
 
-		if (elem != NULL) {
+		if (elem) {
 			g_free(elem->data);
 			opts.snl_list = g_slist_delete_link(opts.snl_list,
 							    elem);
@@ -289,9 +351,9 @@ static int nfctool_snl_cb(guint8 cmd, guint32 idx, gpointer data)
 
 	printf("\n");
 
-	if (opts.snl_list == NULL) {
-		opts.snl = FALSE;
-		nfctool_quit(FALSE);
+	if (!opts.snl_list) {
+		opts.snl = false;
+		nfctool_quit(false);
 	}
 
 	return 0;
@@ -329,7 +391,7 @@ static void sig_term(int sig)
 
 	DBG("Terminating");
 
-	nfctool_quit(TRUE);
+	nfctool_quit(true);
 }
 
 struct nfctool_options opts = {
@@ -339,6 +401,8 @@ struct nfctool_options opts = {
 	.poll_mode = POLLING_MODE_INITIATOR,
 	.device_name = NULL,
 	.adapter_idx = INVALID_ADAPTER_IDX,
+	.enable_dev = FALSE,
+	.disable_dev = FALSE,
 	.set_param = FALSE,
 	.lto = -1,
 	.rw = -1,
@@ -350,49 +414,50 @@ struct nfctool_options opts = {
 	.snap_len = 0,
 	.dump_symm = FALSE,
 	.show_timestamp = SNIFFER_SHOW_TIMESTAMP_NONE,
+	.snep_sap = 0x04,
 	.pcap_filename = NULL,
 };
 
-static gboolean opt_parse_poll_arg(const gchar *option_name, const gchar *value,
+static bool opt_parse_poll_arg(const gchar *option_name, const gchar *value,
 				   gpointer data, GError **error)
 {
-	opts.poll = TRUE;
+	opts.poll = true;
 
 	opts.poll_mode = POLLING_MODE_INITIATOR;
 
-	if (value != NULL) {
+	if (value) {
 		if (*value == 't' || *value == 'T')
 			opts.poll_mode = POLLING_MODE_TARGET;
 		else if (*value == 'b' || *value == 'B')
 			opts.poll_mode = POLLING_MODE_BOTH;
 	}
 
-	return TRUE;
+	return true;
 }
 
-static gboolean opt_parse_set_param_arg(const gchar *option_name,
+static bool opt_parse_set_param_arg(const gchar *option_name,
 					const gchar *value,
 					gpointer data, GError **error)
 {
 	gchar **params = NULL, **keyval = NULL;
 	gchar *end;
 	gint i, intval;
-	gboolean result;
+	bool result;
 
 	params = g_strsplit(value, ",", -1);
 
 	i = 0;
-	while (params[i] != NULL) {
+	while (params[i]) {
 		keyval = g_strsplit(params[i], "=", 2);
 
-		if (keyval[0] == NULL || keyval[1] == NULL) {
-			result = FALSE;
+		if (!keyval[0] || !keyval[1]) {
+			result = false;
 			goto exit;
 		}
 
 		intval = strtol(keyval[1], &end, 10);
 		if (keyval[1] == end) {
-			result = FALSE;
+			result = false;
 			goto exit;
 		}
 
@@ -400,7 +465,7 @@ static gboolean opt_parse_set_param_arg(const gchar *option_name,
 			if (intval < 0 || intval > LLCP_MAX_LTO) {
 				print_error("Bad value: max lto value is %d",
 								LLCP_MAX_LTO);
-				result = FALSE;
+				result = false;
 				goto exit;
 			}
 
@@ -409,7 +474,7 @@ static gboolean opt_parse_set_param_arg(const gchar *option_name,
 			if (intval < 0 || intval > LLCP_MAX_RW) {
 				print_error("Bad value: max rw value is %d",
 								LLCP_MAX_RW);
-				result = FALSE;
+				result = false;
 				goto exit;
 			}
 
@@ -418,17 +483,17 @@ static gboolean opt_parse_set_param_arg(const gchar *option_name,
 			if (intval < 0 || intval > LLCP_MAX_MIUX) {
 				print_error("Bad value: max miux value is %d",
 								LLCP_MAX_MIUX);
-				result = FALSE;
+				result = false;
 				goto exit;
 			}
 
 			opts.miux = intval;
 		} else {
-			result = FALSE;
+			result = false;
 			goto exit;
 		}
 
-		opts.set_param = TRUE;
+		opts.set_param = true;
 
 		g_strfreev(keyval);
 		keyval = NULL;
@@ -436,7 +501,7 @@ static gboolean opt_parse_set_param_arg(const gchar *option_name,
 		i++;
 	}
 
-	result = TRUE;
+	result = true;
 
 exit:
 	if (params)
@@ -448,30 +513,30 @@ exit:
 	return result;
 }
 
-static gboolean opt_parse_show_timestamp_arg(const gchar *option_name,
+static bool opt_parse_show_timestamp_arg(const gchar *option_name,
 					     const gchar *value,
 					     gpointer data, GError **error)
 {
-	if (value != NULL && (*value == 'a' || *value == 'A'))
+	if (value && (*value == 'a' || *value == 'A'))
 		opts.show_timestamp = SNIFFER_SHOW_TIMESTAMP_ABS;
 	else
 		opts.show_timestamp = SNIFFER_SHOW_TIMESTAMP_DELTA;
 
-	return TRUE;
+	return true;
 }
 
-static gboolean opt_parse_snl_arg(const gchar *option_name, const gchar *value,
+static bool opt_parse_snl_arg(const gchar *option_name, const gchar *value,
 				  gpointer data, GError **error)
 {
 	gchar *uri;
 
-	opts.snl = TRUE;
+	opts.snl = true;
 
 	uri = g_strdup(value);
 
 	opts.snl_list = g_slist_prepend(opts.snl_list, uri);
 
-	return TRUE;
+	return true;
 }
 
 static GOptionEntry option_entries[] = {
@@ -484,6 +549,12 @@ static GOptionEntry option_entries[] = {
 	{ "poll", 'p', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK,
 	  opt_parse_poll_arg, "start polling as initiator, target, or both; "
 	  "default mode is initiator", "[Initiator|Target|Both]" },
+	{ "enable", '1', 0, G_OPTION_ARG_NONE, &opts.enable_dev,
+	  "enable device", NULL },
+	{ "disable", '0', 0, G_OPTION_ARG_NONE, &opts.disable_dev,
+	  "disable device", NULL },
+	{ "fw-download", 'w', 0, G_OPTION_ARG_STRING, &opts.fw_filename,
+	  "Put the device in firmware download mode", "fw_filename" },
 	{ "set-param", 's', 0, G_OPTION_ARG_CALLBACK, opt_parse_set_param_arg,
 	  "set lto, rw, and/or miux parameters", "lto=150,rw=1,miux=100" },
 	{ "snl", 'k', 0, G_OPTION_ARG_CALLBACK, &opt_parse_snl_arg,
@@ -495,6 +566,9 @@ static GOptionEntry option_entries[] = {
 	{ "dump-symm", 'y', 0, G_OPTION_ARG_NONE, &opts.dump_symm,
 	  "dump SYMM packets to stdout (flooding); only relevant with -n",
 	  NULL },
+	{ "snep-sap", 'e', 0, G_OPTION_ARG_INT, &opts.snep_sap,
+	  "Specify the sap number to be used for snep decoding; "
+	  "only relevant with -n", "0x04" },
 	{ "show-timestamp", 't', G_OPTION_FLAG_OPTIONAL_ARG,
 	  G_OPTION_ARG_CALLBACK, opt_parse_show_timestamp_arg,
 	  "show packet timestamp as the delta from first frame (default) "
@@ -529,7 +603,7 @@ static int nfctool_options_parse(int argc, char **argv)
 		goto done;
 	}
 
-	if (opts.device_name != NULL) {
+	if (opts.device_name) {
 		if (strncmp("nfc", opts.device_name, 3) != 0) {
 			print_error("Invalid device name: %s",
 							opts.device_name);
@@ -547,8 +621,14 @@ static int nfctool_options_parse(int argc, char **argv)
 		}
 	}
 
-	opts.need_netlink = opts.list || opts.poll ||
-			    opts.set_param || opts.snl;
+	if (opts.enable_dev || opts.disable_dev)
+		opts.list = true;
+
+	if (opts.poll)
+		opts.enable_dev = true;
+
+	opts.need_netlink = opts.list || opts.poll || opts.set_param ||
+			    opts.snl || opts.fw_filename;
 
 	if (!opts.need_netlink && !opts.sniff) {
 		printf("%s", g_option_context_get_help(context, TRUE, NULL));
@@ -556,7 +636,8 @@ static int nfctool_options_parse(int argc, char **argv)
 		goto exit;
 	}
 
-	if ((opts.poll || opts.set_param || opts.sniff || opts.snl) &&
+	if ((opts.poll || opts.set_param || opts.sniff || opts.snl ||
+	    opts.enable_dev || opts.disable_dev || opts.fw_filename) &&
 	    opts.adapter_idx == INVALID_ADAPTER_IDX) {
 		print_error("Please specify a device with -d nfcX option");
 
@@ -588,22 +669,25 @@ static void nfctool_main_loop_start(void)
 
 static void nfctool_options_cleanup(void)
 {
-	if (opts.device_name != NULL)
+	if (opts.device_name)
 		g_free(opts.device_name);
 
-	if (opts.pcap_filename != NULL)
+	if (opts.pcap_filename)
 		g_free(opts.pcap_filename);
 
+	if (opts.fw_filename != NULL)
+		g_free(opts.fw_filename);
+
 	g_slist_free_full(opts.snl_list, g_free);
 }
 
 static void nfctool_main_loop_clean(void)
 {
-	if (main_loop != NULL)
+	if (main_loop)
 		g_main_loop_unref(main_loop);
 }
 
-static void nfctool_quit(gboolean force)
+static void nfctool_quit(bool force)
 {
 	if (force || (!opts.sniff && !opts.snl))
 		g_main_loop_quit(main_loop);
@@ -632,12 +716,29 @@ int main(int argc, char **argv)
 			goto exit_err;
 	}
 
+	if (opts.fw_filename) {
+		err = nfctool_fw_download(opts.fw_filename);
+		if (err)
+			goto exit_err;
+
+		goto start_loop;
+	}
+
 	if (opts.sniff) {
 		err = sniffer_init();
 		if (err)
 			goto exit_err;
 	}
 
+	if (opts.enable_dev || opts.disable_dev) {
+		err = nfctool_set_powered(opts.enable_dev);
+
+		if (err && err != -EALREADY)
+			goto exit_err;
+
+		err = 0;
+	}
+
 	if (opts.list && !opts.set_param)
 		adapter_idx_print_info(opts.adapter_idx);
 
@@ -660,7 +761,8 @@ int main(int argc, char **argv)
 	if (opts.snl)
 		nfctool_snl();
 
-	if (opts.poll || opts.sniff || opts.snl)
+start_loop:
+	if (opts.poll || opts.sniff || opts.snl || opts.fw_filename)
 		nfctool_main_loop_start();
 
 done:
@@ -677,5 +779,8 @@ exit_err:
 
 	nfctool_options_cleanup();
 
+	if (err)
+		print_error("%s", strerror(-err));
+
 	return err;
 }
